# assign_SNPs.R
# This script creates a file that assigns variant sites to a particular chromosomal location, using information in a gtf file
# It uses a gtf file for a given species and the vcf file generated from the variant calling pipeline 

require("intervals")

#reading in GTF file
sec.exons<-read.table("/Volumes/Promise Pegasus/emily/copy_data/sim_sec/annotations/dsec-all-r1.3.gtf.gz",as.is=TRUE,sep="\t")
#need to separate based on semicolon too

sec.exons<-sec.exons[sec.exons$V3=="exon",]
tmp<-sapply(sec.exons$V9,strsplit,";")
tmp<-sapply(sec.exons$V9,function(x){unlist(strsplit(x,";"))})
tmp<-t(tmp)
rownames(tmp)<-NULL
sec.exons<-sec.exons[,-9]
sec.exons<-cbind(sec.exons,tmp)


##add column names
colnames(sec.exons)<-c("chr","fly","ex","pos.l","pos.r","dot","sign","dot2","gene_id","gene_symbol","transcript_id","transcript_symbol")

sec.genes<-sec.exons$gene_id[!duplicated(sec.exons$gene_id)]  ##remove duplicated exons


###Load in the called SNP file, VCF file from GATK pipeline
snps<-read.table("/Volumes/Promise Pegasus/emily/copy_data/mau_sec/mau_to_sec/var_calls_mau_to_sec/mau_to_sec_union_singlevar.vcf",as.is=TRUE)  

#snps<-snps[,-3]##remove .
snps<-snps[,c(1,2,4,5,6)]

 snps$loc<-paste(snps$V1,snps$V2,sep="_")  ##attach chromosome label to physical location to make unique id (separate them by "_")
colnames(snps)<-c("chr","pos","allele0","allele1","Qual","loc")  #,"infoind1","infoind2","geno1","geno2"

fileout<-"SNPs_by_transcript.txt"
system(paste("rm ",fileout)) ##remove any pre-existing file

gene.allele.spec<-sapply(sec.genes,function(gene){
        gene.name<-strsplit(as.character(gene)," ")[[1]][2]
                                        #gene.name<-strsplit(gene," ") 
        these.exons <-sec.exons[sec.exons$gene_id==gene,]
        my.exons<-as.data.frame(interval_union(Intervals(these.exons[,c("pos.l","pos.r")])))
        colnames(my.exons)<-c("pos.l","pos.r")
        my.exons$chr<-these.exons$chr[1]
        #recover()
        apply(my.exons,1,function(exon){
             these.snps<- which(snps$chr==as.character(exon["chr"]) & snps$pos>=as.numeric(exon["pos.l"]) & snps$pos< as.numeric(exon["pos.r"]))
             #recover()
             if(length(these.snps)) write.table(file=fileout,cbind(snps[these.snps,],gene.name),append=TRUE,quote=FALSE,col.names=FALSE,row.names=FALSE)  #print(cbind(snps[these.snps,],gene)) 
            })
})        
