# This script organizes count information from the output of the pysam script (File_S2.py)
##It sorts the sim and sec reads for each SNP and each sample when in bam_RNA_snps

library("intervals")

# reading in GTF file
sec.exons<-read.table("/Volumes/Promise Pegasus/emily/copy_data/sim_sec/annotations/dsec-all-r1.3.gtf.gz",as.is=TRUE,sep="\t")
#need to separate based on semicolon too

# pulling out exons from GTF file
sec.exons<-sec.exons[sec.exons$V3=="exon",]
tmp<-sapply(sec.exons$V9,strsplit,";")
tmp<-sapply(sec.exons$V9,function(x){unlist(strsplit(x,";"))})
tmp<-t(tmp)
rownames(tmp)<-NULL
sec.exons<-sec.exons[,-9]
sec.exons<-cbind(sec.exons,tmp)


##add column names
colnames(sec.exons)<-c("chr","fly","ex","pos.l","pos.r","dot","sign","dot2","gene_id","gene_symbol","transcript_id","transcript_symbol")

sec.genes<-sec.exons$gene_id[!duplicated(sec.exons$gene_id)]  ##remove duplicated exons


###Load in the called SNP file 
all.snps<-read.table("/Volumes/Promise Pegasus/emily/copy_data/mau_sec/mau_to_sec/var_calls_mau_to_sec/mau_to_sec_union_singlevar.vcf",as.is=TRUE)  

#all.snps<-all.snps[,-3]##remove .
all.snps<-all.snps[,c(1,2,4,5,6)]

 all.snps$loc<-paste(all.snps$V1,all.snps$V2,sep="_")  ##attach chromosome label to physical location to make unique id (separate them by "_")
colnames(all.snps)<-c("chr","pos","allele0","allele1","Qual","loc")  #,"infoind1","infoind2","geno1","geno2"

files=dir(path="sec_variants_bamfiles_thinned/",pattern="*.snps") 
#files="sec_variants_bamfiles/50_50_st5_2_sim.bam.snps"
#files=dir(pattern="sec_variants_bamfiles_thinned/"*.snps")  ###get the names of all of the .snp files in the directory.


###First make list of SNPs to remove, due to having deletions
remove<-character()
for(file in files){  ###Loop over individuals?
        ###read in the SNP file for an individual
        indsnps<-read.table(paste("sec_variants_bamfiles_thinned/",file,sep=""),as.is=TRUE)
        ##attach chromosome label to physical location to make unique id (separate them by "_")
        indsnps$loc<-paste(indsnps$V1,indsnps$V3,sep="_")

        colnames(indsnps)<-c(colnames(indsnps)[1:6],c('A', 'C', 'G', 'T', 'N', 'D'),"loc")    ##new column names  
        remove<-c(remove,indsnps$loc[indsnps$D!=0])
		cat(file,mean(indsnps$D!=0),mean(indsnps$D[indsnps$D!=0]),"\n")
		#tmp<-cbind(tmp,indsnps[,c("loc","D")])
}

##mean number of times each removal gene appears
length(remove)/length(remove[!duplicated(remove)])
##how many times does each SNP appear in removal list
table(table(remove))
##what rough fraction of SNPs are in removal list
length(remove[!duplicated(remove)])/nrow(indsnps)

remove<-remove[!duplicated(remove)]

for(file in files){  ###Loop over individuals
		print(file)
        ###read in the SNP file for an individual
        indsnps<-read.table(paste("sec_variants_bamfiles_thinned/",file,sep=""),as.is=TRUE)
        ##attach chromosome label to physical location to make unique id (separate them by "_")
        indsnps$loc<-paste(indsnps$V1,indsnps$V3,sep="_")
		
        colnames(indsnps)<-c(colnames(indsnps)[1:6],c('A', 'C', 'G', 'T', 'N', 'D'),"loc")    ##new column names  
    
    ###Taken out 1 March 2018, dont remove SNPS with deletions.    
 ###       indsnps<-indsnps[!(indsnps$loc %in% remove),]
        
        snps<-merge(x=all.snps,y=indsnps[,5:13],by.x="loc",by.y="loc") ##merge all.snps and the individual SNPs table
        
        ###this function makes sure that we are giving the reference allele
        calls<-apply(snps,1,function(snp){
 #               if(snp["geno1"]=="0/0"){        
                        return(c(as.numeric(snp[snp["allele0"]]),as.numeric(snp[snp["allele1"]])))
  #              }else{
   #                     return(c(as.numeric(snp[snp["allele1"]]),as.numeric(snp[snp["allele0"]])))
  #              }
        })
        
        calls<-t(calls)
        colnames(calls)<-c("moms","dads")
        snps<-cbind(snps,calls)
        
    ###write out the file, for each ind, where for each SNP we have polarized the alleles into mom/dad
        write.table(file=paste(file,".momdad",sep=""),snps,col.names=FALSE,row.names=FALSE,quote=FALSE)
        
        ###now run for all of the genes. 
        ##For each gene we get all the exons. 
        
        gene.allele.spec<-sapply(sec.genes,function(gene){
                these.exons <-sec.exons[sec.exons$gene_id==gene,]
                my.exons<-as.data.frame(interval_union(Intervals(these.exons[,c("pos.l","pos.r")])))
		colnames(my.exons)<-c("pos.l","pos.r")
                my.exons$chr<-these.exons$chr[1]
                exon.count<-apply(my.exons,1,function(exon){
  
                                these.snps<- snps$chr==as.character(exon["chr"]) & snps$pos>=as.numeric(exon["pos.l"]) & snps$pos< as.numeric(exon["pos.r"])  ##the SNPs that fall in exon
                                if(sum(these.snps)==0) return(rep(0,3))  ##if no SNPS
                                ##if there are SNPs, we sum their mom and dad counts.
                                return(c(sum(snps[these.snps,c("moms")]),sum(snps[these.snps,c("dads")]),sum(these.snps)))
                        })
                return(colSums(t(exon.count)))
        }
        )
        
##Write out the file of per gene counts for an individual   
    write.table(file=paste(file,".pergene",sep=""),cbind(as.character(sec.genes),t(gene.allele.spec)),col.names=FALSE,row.names=FALSE,quote=FALSE)

}


